<?php

/*********************************************************************************
 * The contents of this file are subject to the GNU General Public License
 * ("License"); You may not use this file except in compliance with the License
 * Written by: Nolan Andres, PeaceWorks Computer Consulting Inc.
 * Copyright (C) Sutherland-Schultz Inc.
 *     www.sutherland-schultz.com 
 * All Rights Reserved.
 ********************************************************************************/

require_once('include/database/PearDatabase.php');

class CustomField {

    var $id;
    var $sequence;
    var $block;
    var $tab;
    var $position;
    var $subtab;
    var $subblock;
	var $fieldlabel; //Added by Wesley Eby
	var $fieldname; // Added by Wesley Eby
    
    function CustomField() {
    }
    
    function clear() {
        $this->id = 0;
        $this->tab = 0;
        $this->block = 0;
        $this->sequence = 0;
        $this->subtab = 0;
        $this->subblock = 0;
        $this->position = -1;
		$this->fieldlabel = NULL; //Added by Wesley Eby
		$this->fieldname=NULL; //Added by Wesley Eby
    }
    
    function loadFromDB( $fieldid ) {
        global $adb;
        
        $sql = 'select tabid, block, sequence, fieldlabel, fieldname from field where fieldid=' . intval($fieldid);
        $result = $adb->query($sql);

        if ( $adb->num_rows($result) > 0 ) {
            $rec = $adb->fetch_array($result,0);
            $this->id = $fieldid;
            $this->tab = $rec['tabid'];
            $this->block = $rec['block'];
            $this->sequence = $rec['sequence'];
            $this->subtab = $this->getSubtabID($rec['sequence']);
            $this->subblock = $this->getSubblockID($rec['sequence']);
			$this->fieldlabel = $rec['fieldlabel']; //added by Wesley Eby
			$this->fieldname = $rec['fieldname']; //added by Wesley Eby
            return 1; 
        } else {
            $this->clear();
            return 0;
        }
        
        // set the position and sequence using the getMyBlock function, and throw away the block fields.
        // seems a little kludgy, but it makes sense in at least one respect.
        // this is not a persistent object class...so when we do the move, we can 
        // only reasonably really assume that the fieldid, tab, and block are going to
        // stay constant. getMyBlock re-sets sequence and position so that it's accurate,
        // and does the move within a transaction to avoid race conditions. 
        $throwaway = $this->getMyBlockFields( $adb );
    }
    
    function moveUp() {
        global $adb;
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        if ( $this->position > 0 ) {
            $sql = 'update field set sequence = '.$this->sequence.' where fieldid = '.$fldarr[$this->position-1]['fieldid'];
            $adb->query($sql);
            $sql = 'update field set sequence = '.$fldarr[$this->position-1]['sequence'].' where fieldid = '.$this->id;
            $adb->query($sql);
            $this->position--;
            $this->sequence = $fldarr[$this->position-1]['sequence'];
            $this->subtab = $this->getSubtabID($this->sequence);
            $this->subblock = $this->getSubblockID($this->sequence);
        }
        $adb->completeTransaction();
    }
    
    function moveDown() {
        global $adb;
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );

        if ( $this->position < (sizeof($fldarr)-1) ) {
            $sql = 'update field set sequence = '.$this->sequence.' where fieldid = '.$fldarr[$this->position+1]['fieldid'];
            $adb->query($sql);
            $sql = 'update field set sequence = '.$fldarr[$this->position+1]['sequence'].' where fieldid = '.$this->id;
            $adb->query($sql);
            $this->position++;
            $this->sequence = $fldarr[$this->position+1]['sequence'];
            $this->subtab = $this->getSubtabID($this->sequence);
            $this->subblock = $this->getSubblockID($this->sequence);
        }
        $adb->completeTransaction();
    }


    // MOVEMENT WITH RESPECT TO SUBBLOCKS
    function movePrevSubblock() {
        global $adb;

        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        if ( $this->subblock > 0 ) {
            $newseq = $this->getNextSequence( $fldarr, $this->subtab, $this->subblock-1 );
            $sql = "update field set sequence = $newseq where fieldid = $this->id";
            $this->sequence = $newseq;
            $this->subblock--;
            $adb->query($sql);
        }

        $adb->completeTransaction();
    }
    
    function moveNextSubblock() {
        global $adb;
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );

        $maxblock = $this->getMaxSubblockID($fldarr, $this->subtab);
        if ( $this->subblock < $maxblock ) {
            $newseq = $this->getNextSequence( $fldarr, $this->subtab, $this->subblock+1 );
            $sql = "update field set sequence = $newseq where fieldid = $this->id";
            $this->sequence = $newseq;
            $this->subblock++;
            $adb->query($sql);
        }

        $adb->completeTransaction();
    }
    
    function moveNewSubblock() {
        global $adb;

        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        $maxblock = $this->getMaxSubblockID($fldarr, $this->subtab);
        $newseq = $this->getNextSequence( $fldarr, $this->subtab, $maxblock+1 );
        $newsubblockid = $this->getSubblockID($newseq);
        $sql = "update field set sequence = $newseq where fieldid = $this->id";
        $adb->query($sql);
        $sql = "delete from subblock where tabid=$this->tab and blockid=$this->block and subtabid=$this->subtab and subblockid=$newseq";
        $adb->query($sql);
        
        $adb->completeTransaction();
    }
    
    function renameMySubblock( $newname ) {
        global $adb;

        $newname = $this->SQLSafe($newname);
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        $this->renameSubblock( $adb, $this->tab, $this->block, $this->subtab, $this->subblock, $newname);        
        $adb->completeTransaction();    
    }

    function renameSubblock( $adb, $tabid, $block, $subtab, $subblock, $newname ) {
        $newname = str_replace("'","''",$newname);
        $sql = "delete from subblock where tabid=$tabid and block=$block and subtabid=$subtab and subblockid=$subblock";
        $adb->startTransaction();
        $adb->query($sql);
        $sql = "insert into subblock values($tabid,$block,$subtab,$subblock,'$newname')";
        $adb->query($sql);
        $adb->completeTransaction();    
    }

    // MOVEMENT WITH RESPECT TO SUBTABS
    function movePrevSubtab() {
        global $adb;

        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        if ( $this->subtab > 0 ) {
            $newseq = $this->getNextSequence( $fldarr, $this->subtab-1 );
            $sql = "update field set sequence = $newseq where fieldid = $this->id";
            $this->sequence = $newseq;
            $this->subtab--;
            $this->subblock = 0;
            $adb->query($sql);
        }

        $adb->completeTransaction();
    }
    
    function moveNextSubtab() {
        global $adb;
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );

        $maxtab = $this->getMaxSubtabID($fldarr);
        if ( $this->subtab < $maxtab ) {
            $newseq = $this->getNextSequence( $fldarr, $this->subtab+1 );
            $sql = "update field set sequence = $newseq where fieldid = $this->id";
            $this->sequence = $newseq;
            $this->subtab++;
            $this->subblock = 0;
            $adb->query($sql);
        }

        $adb->completeTransaction();
    }
    
    function moveNewSubtab() {
        global $adb;

        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        $maxtab = $this->getMaxSubtabID($fldarr);
        $newseq = $this->getNextSequence( $fldarr, $maxtab+1 );
        $sql = "update field set sequence = $newseq where fieldid = $this->id";
        $adb->query($sql);
        $sql = "delete from subtab where tabid=$this->tab and blockid=$this->block and subtabid=$this->subtab";
        $adb->query($sql);
        
        $adb->completeTransaction();
    }
    
    function renameMySubtab( $newname ) {
        global $adb;

        $newname = $this->SQLSafe($newname);
        
        $adb->startTransaction();
        $fldarr = $this->getMyBlockFields( $adb );
        
        $this->renameSubtab( $adb, $this->tab, $this->block, $this->subtab, $newname);        
        $adb->completeTransaction();    
    }

    function renameSubtab( $adb, $tabid, $block, $subtab, $newname ) {
        $newname = str_replace("'","''",$newname);
        $adb->startTransaction();
        $sql = "delete from subtab where tabid=$tabid and block=$block and subtabid=$subtab";
        $adb->query($sql);
        $sql = "insert into subtab values($tabid,$block,$subtab,'$newname')";
        $adb->query($sql);
        $adb->completeTransaction();    
    }

    // getMyBlockFields
    //
    // retrieves an array of array(fieldid,sequence) representing this custom field's block
    // SIDE-EFFECT: $this->position, $this->sequence 
    //
    function getMyBlockFields( $adb ) {
        return $this->getBlockFields( $adb, $this->tab, $this->block );
    }    
    
    function getBlockFields( $adb, $tab, $block ) {
        $sql = "select fieldid, sequence from field where tabid = $tab and block = $block order by sequence";
        $result = $adb->query($sql);
        
        $this->position = -1;
        $fldarr = array();
        for ( $i = 0; $i < $adb->num_rows($result); $i++) {
            $rec = $adb->fetch_array($result,$i);
            if ( $rec['fieldid'] == $this->id ) {
                $this->position = $i;
                $this->sequence = $rec['sequence'];
                $this->subtab = $this->getSubtabID($rec['sequence']);
                $this->subblock = $this->getSubblockID($rec['sequence']);
           }
            $fldarr[] = $rec;
        }
        
        if ( $this->position < 0 ) {
            $this->clear();
        }
        return $fldarr;
    }


    // FUNCTIONS REQUIRING A blockarray
    
    // getNextSequence( $fieldblockarray, $subtab )
    //
    // returns the next sequence number available for the given subtab and subblock in the given field block
    function getNextSequence( $fieldblockarray, $subtab=0, $subblock=0 ) {
        $minseq = $subtab * 10000 + $subblock * 100;
        $maxseq = $minseq + 99;

        // if the minimum seq number on this subtab is already larger than the biggest we have, 
        // then just return the min seq number (we're starting a new tab). Otherwise, flip through 
        // the array til we get to the highest sequence on the subtab, and add one to it
        if ( $fieldblockarray[sizeof($fieldblockarray)-1]['sequence'] < $minseq ) {
            return $minseq;
        } else {        
            $i = 0;
            while ( ($i < sizeof($fieldblockarray)) && ($fieldblockarray[$i]['sequence'] < $maxseq) )
                $i++;
            if ( $fieldblockarray[$i-1]['sequence'] < $minseq ) {
                return $minseq;
            } else {
                return $fieldblockarray[$i-1]['sequence'] + 1;
            }
        }
    }
    
    function getMinSubtabID( $fieldblockarray ) {
        return $this->getSubtabID($fieldblockarray[0]['sequence']);
    }
    
    function getMaxSubtabID( $fieldblockarray ) {
        return $this->getSubtabID($fieldblockarray[sizeof($fieldblockarray)-1]['sequence']);
    }
    
    function getMinSubblockID( $fieldblockarray, $subtab ) {
        $minseq = $subtab * 10000;
        
        $i = 0;
        while ( ($i < sizeof($fieldblockarray)) && ($fieldblockarray[$i]['sequence'] < $minseq) )
                $i++;
        if ( $i > 0 ) {
            if ( $i == sizeof($fieldblockarray) ) {
                return 0;
            } else {
                return $this->getSubblockID($fieldblockarray[$i-1]['sequence']);
            }
        } else {
            return 0;
        }
    }
    
    function getMaxSubblockID( $fieldblockarray, $subtab ) {
        $minseq = $subtab * 10000;
        $maxseq = $minseq + 9999;
        
        if ( $fieldblockarray[sizeof($fieldblockarray)-1]['sequence'] < $minseq ) {
            return 0;
        } else {        
            $i = 0;
            while ( ($i < sizeof($fieldblockarray)) && ($fieldblockarray[$i]['sequence'] < $maxseq) )
                $i++;
            return $this->getSubblockID($fieldblockarray[$i-1]['sequence']);
        }
    }
    
    function getSubtabID( $seq ) {
        return floor($seq/10000);
    }
    
    function getSubblockID( $seq ) {
        return floor(($seq % 10000)/100);
    }
	
	
	//MOD: Wesley Eby
	// This function allows the label of a custom field to be modified throughout the database.
	function editPersistentCustomFieldLabel( $newFldLabel )  {
		global $adb;
		
		$adb->startTransaction();
		
		//Ensure that the new fieldlabel is not already in use in the current module.
		$checkquery="select * from field where tabid='".$this->tab."'and fieldlabel='".$newFldLabel."'";
		$checkresult=$adb->query($checkquery);
		$success=true;
		if($adb->num_rows($checkresult) != 0) {
			$success=false;
		}
		else {
			//Modify field table
			$sql= "UPDATE field SET fieldlabel='$newFldLabel' where fieldid = $this->id";
			$adb->query($sql);
			$oldFldLabel = $this->fieldlabel;
			$this->fieldlabel = $newFldLabel;
			
			$newFldLabelNoSpace = str_replace(" ","_",$newFldLabel);

			//Note:   reportsummary columnname follows a different format, so can't use updateDBTable()
			$sql="SELECT columnname FROM reportsummary where fieldid = $this->id";
			$result = $adb->query($sql);
			$noofrows = $adb->num_rows($result);
			for($i=0; $i<$noofrows; $i++)
			{
				$oldColName = $adb->query_result($result,$i,"columnname");
				$explodedString = explode(':', $oldColName, 5);
				$explodedString[3] = substr_replace( $explodedString[3], $newFldLabelNoSpace, 0, strlen($oldFldLabel) );
				$newColName = implode(':', $explodedString);
				if ($newColName) {
					$sql = "UPDATE reportsummary SET columnname='$newColName' where columnname='$oldColName'";
					$adb->query($sql);
				}
				else  {
					$success = false;
				}
			}
			
			//change cvcolumnlist
			$sql="SELECT columnname FROM cvcolumnlist where columnname like '%:%:".$this->fieldname.":%'";
			$result = $adb->query($sql);
			$noofrows = $adb->num_rows($result);
			for($i=0; $i<$noofrows; $i++)
			{
				$oldColName = $adb->query_result($result,$i,"columnname");
				$newColName = replaceFieldLabelInColName($oldColName, $newFldLabelNoSpace, 3);
				if ($newColName) {
					$sql = "UPDATE cvcolumnlist SET columnname='$newColName' where columnname='$oldColName'";
					$adb->query($sql);
				}
				else  {
					$success = false;
				}
			}
			
			//change cvadvfilter
			$sql="SELECT columnname FROM cvadvfilter where columnname like '%:%:".$this->fieldname.":%'";
			$result = $adb->query($sql);
			$noofrows = $adb->num_rows($result);
			for($i=0; $i<$noofrows; $i++)
			{
				$oldColName = $adb->query_result($result,$i,"columnname");
				$newColName = replaceFieldLabelInColName($oldColName, $newFldLabelNoSpace, 3);
				if ($newColName) {
					$sql = "UPDATE cvadvfilter SET columnname='$newColName' where columnname='$oldColName'";
					$adb->query($sql);
				}
				else  {
					$success = false;
				}
			}
			
			//change reportgevis - this must occur before select column is changed
			$sql="SELECT queryid, columnname FROM selectcolumn where fieldid = $this->id";
			$result = $adb->query($sql);
			$noofrows = $adb->num_rows($result);
			for($i=0; $i<$noofrows; $i++)
			{
				$queryid = $adb->query_result($result, $i, "queryid");
				$sql="SELECT mappings FROM reportgevis where reportid=$queryid";
				$mappings = $adb->getOne($sql);
				
				if ($mappings)  {
					$searchString = $adb->query_result($result,$i,"columnname");
					$explodedString = explode(':', $searchString, 4);
					$searchString = '$['.$explodedString[2].']';
					$explodedString = explode('_', $searchString, 2);
					$explodedString[1] = $newFldLabelNoSpace.']';
					$replacementString = implode('_', $explodedString);
					
					$gevalues = unserialize($mappings);
					//print_r($gevalues);
					foreach($gevalues as $key=>$value)  {
							$gevalues[$key] = str_replace($searchString, $replacementString, $value);
					}
					//print_r($gevalues);

					$mappings = serialize($gevalues);
					$sql = "UPDATE reportgevis SET mappings='$mappings' where reportid=$queryid";
					$adb->query($sql);
				}
				else  {
					$success = false;
				}
			}
			
			$success = $success && updateDBTable('relcriteria', $this->id, $newFldLabelNoSpace);
			$success = $success && updateDBTable('selectcolumn', $this->id, $newFldLabelNoSpace);  //updating selectcolumn must happen after update to reportgevis
			$success = $success && updateDBTable('reportsortcol', $this->id, $newFldLabelNoSpace);

				
		}
		$adb->completeTransaction();
		return $success;
	}
} //end of class
	
	//This function expects a string containing a list of ':' separated values.  The labelPos postion (base 0) contains
	//the table name and field label separated by an '_'.  If the values are not correctly separated, or the field
	//label is not correct, then return false.  Otherwise return the string with the field label being replaced.
	function replaceFieldLabelInColName($oldColName, $newFldLabel, $labelPos=2)
	{
		$explodedColName = explode(':', $oldColName, $labelPos+2);
		if ( count ($explodedColName) != $labelPos+2 ) {
			return false; //the column format of ':' separated values is not as expected
		}
		$explodedLabel = explode('_', $explodedColName[$labelPos], 2);
		if ( count($explodedLabel) != 2 ) {
			return false;
		}
		
		$explodedLabel[1] = $newFldLabel;
		
		$explodedColName[$labelPos] = implode('_', $explodedLabel);
		$newColName = implode(':', $explodedColName);
		return $newColName;
	}
	
	//The passed in table must store the fieldid in a column called fieldid and the column name
	// in a column called columnname.  Also note that this function calles replaceFieldLabelInColName, so
	// it has the same data restrictions as those with that function.
	function updateDBTable($tablename, $fieldid, $newFldLabelNoSpace)
	{
		$success = true;
		global $adb;
		
		$sql="SELECT columnname FROM $tablename where fieldid = $fieldid";
		$result = $adb->query($sql);
		$noofrows = $adb->num_rows($result);
		for($i=0; $i<$noofrows; $i++)
		{
			$oldColName = $adb->query_result($result,$i,"columnname");
			$newColName = replaceFieldLabelInColName($oldColName, $newFldLabelNoSpace);
			if ($newColName) {
				$sql = "UPDATE $tablename SET columnname='$newColName' where columnname='$oldColName'";
				$adb->query($sql);
			}
			else  {
				$success = false;
			}
		}
		return $success;
	}
    //END MOD
?>