<?php
/*********************************************************************************
** The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
*
 ********************************************************************************/

require_once('include/utils.php');
require_once('include/database/PearDatabase.php');
require_once('modules/Settings/CustomField.php');

function getBlockInformation($module, $block, $mode, $col_fields)
{
    // NOLAN: throw away $mode parameter
    //  - the original code, prior to factoring this and the next function into getFieldBlock
    //    did not use the parameter at all. so even though getFieldBlock has a $mode parameter,
    //    there's no reason to believe that the passed-in value here is useful, and every reason
    //    to believe that it's not.
	return getFieldBlock($module, $block, $col_fields, 'edit');
}


function getDetailBlockInformation($module, $block, $col_fields, $omit_fields = Array(), $subtab = '')
{
    return getFieldBlock($module, $block, $col_fields, 'detail', $omit_fields, $subtab);
}

function getFieldBlockColumns($mode, $tabid, $displaytypes = '=1', $block = '', $subtab = '') {

	global $adb;
    global $profile_id;
    	
	if ($mode == 'saveCRMEntity') {
		$modestring = "or field.typeofdata like '%~M%'";
		$tabfields = '';
		$tabjoin = '';
	} else {
		$modestring = '';
		$tabfields = ', subtab.subtabid, subtab.subtabstring, subblock.subblockid, subblock.subblockstring';
		$tabjoin = 'left join subtab 
                on field.tabid = subtab.tabid and field.block = subtab.block 
                    and floor(field.sequence/10000)=subtab.subtabid 
                left join subblock 
                on field.tabid = subblock.tabid and field.block = subblock.block 
                    and subtab.subtabid=subblock.subtabid and floor(mod(field.sequence,10000)/100)=subblock.subblockid 
               ';
	}
	
	if ( $block == '') { 
		$blockstring = '';
	} else {
		$blockstring = "and field.block = $block";
	}
  if (strcmp($subtab, '') == 0) {
    $subtab_where = '';
  } else {
    $subtab_where = ' and subtab.subtabid = ' . intval($subtab);
  }
  
	//retreive the fields from database
    
    $sql = "select distinct field.* $tabfields
            from field 
                $tabjoin
                 inner join profile2field 
                on profile2field.fieldid=field.fieldid and field.tabid=profile2field.tabid
                inner join def_org_field 
                on def_org_field.fieldid=field.fieldid and field.tabid=profile2field.tabid
            where field.tabid=".$tabid." $blockstring and field.displaytype".$displaytypes." and (profile2field.visible=0 $modestring) 
                and def_org_field.visible=0 and profile2field.profileid=".$profile_id." $subtab_where order by sequence";
	//echo $sql . '<br/><br/>';
	$result = $adb->query($sql);
	return $result;
}

function getFieldBlock($module, $block, $col_fields, $mode, $omit_fields=Array(), $subtab = '')
{
	//echo "function getFieldBlock($module, $block, $col_fields, $mode)\ncol_fields=";
	//print_r($col_fields);
    // MOD: Nolan Andres
    // getDetailBlockInformation and getBlockInformation refactored due to so much duplicated code, 
    // then modified significantly to include tabpane based on sequence number
    
	//retreive the tabid	
	global $adb;
	$tabid = getTabid($module);
    global $mod_strings;
    $cf = new CustomField();

    // set flags
    $moveAddress = '';

    if ( $mode == 'edit' ) {
        $displaytypes = '=1';
        if (($module == 'Accounts' || $module == 'Contacts' || $module == 'Quotes' || $module == 'Orders' || $module == 'SalesOrder'|| $module == 'Invoice') && $block == 2)
        {
            global $vtlog;
            $vtlog->logthis('module is '.$module,'info');          
            $moveAddress = "<td rowspan='5' valign='middle' align='center'><input title='Copy billing address to shipping address'  class='button' onclick='return copyAddressRight(EditView)'  type='button' name='copyright' value='&raquo;' style='padding:0px 2px 0px 2px;font-size:12px'><br><br>
                <input title='Copy shipping address to billing address'  class='button' onclick='return copyAddressLeft(EditView)'  type='button' name='copyleft' value='&laquo;' style='padding:0px 2px 0px 2px;font-size:12px'></td>";
        }
    } else {
        $displaytypes = ' in (1,2)';
    }

	$result = getFieldBlockColumns($mode, $tabid, $displaytypes, $block, $subtab);
	$noofrows = $adb->num_rows($result);

    // read the results into an array for processing
    $fldarr = array();
    for ( $i = 0; $i < $noofrows; $i++) {
        $fldarr[] = $adb->fetch_array($result,$i);
    }

    // check on subtabbing
    if ( count($fldarr) > 0 ) {
        $maxtab = $cf->getMaxSubtabID($fldarr);
        $mintab = $cf->getMinSubtabID($fldarr);
    } else {
        $mintab = 0;
        $maxtab = 0;
    }
    if ($mintab != $maxtab) {
        $tabbing = true;
    } else {
        $tabbing = false;
    }
    $cursubtab = $mintab;
    $cursubtabstring = '';

    $minsubblock = $cf->getMinSubblockID($fldarr, $cursubtab);
    $maxsubblock = $cf->getMaxSubblockID($fldarr, $cursubtab);
    $cursubblock = $minsubblock;
    $cursubblockstring = '';
    if ( !$tabbing && ($minsubblock == $maxsubblock) ) {
        $subblocking = false;
    } else {
        $subblocking = true;
    }
    
    $firstinline = false;
	$output='';
    
    if ( $tabbing ) {
        $output .= '<script type="text/javascript" src="include/tabpane/js/tabpane.js"></script>
                    <style type="text/css">@import url("include/tabpane/css/tab.webfx.css");</style>
                    <tr><td><div class="tab-pane" id="tab-pane-' . $block . '">
                    <script type="text/javascript">
                    tp' . $block . ' = new WebFXTabPane( document.getElementById( "tab-pane-' . $block . '" ) );
                    var tab_to_id = new Array();
                    var next_tab_num = 0;
                    </script>'."\n";
    }
    
    $i = 0;
    foreach($fldarr as $row)
    {
      if(!empty($omit_fields)) {
    		if ( array_search( trim($row['fieldname']), $omit_fields) !== false ) {
    			continue;
    		}
      }
        $firstinline = !$firstinline;
        $sequence = $row['subtabid'];
        // get the new sequence number; if it starts a new tab, then create the output for it
        if ( !$subblocking || ($i != 0)  && ($cursubtab == $row['subtabid']) && ($cursubblock == $row['subblockid']) ) {
            $newsubblock = false;
            $newsubtab = false;
        } else {
            $newsubblock = true;
            $subblocking = true;
            if ($cursubtab != $row['subtabid'] || $i == 0) {
                $newsubtab = $tabbing;
            }
        }

        if ( $newsubblock ) {
            // end the previous subblock (unless we're just starting, so there isn't one)
            if ( $i > 0 ) {
                if ( !$firstinline ) {
                    // second item in this row is empty...put in an empty cell
                    $output .= "<td></td></tr>\n";
                }
                $output .= "</table><br/></td></tr>\n";
            }
            if ( $newsubtab ) {
                // end the previous subtab (again, unless we're just starting)
                if ( $i > 0 ) {
                    if ( !$firstinline ) {
                        // second item in this row is empty
                        $output .= "<td></td></tr>\n";
                    }
                    $output .= "</table></div>\n";
                }
            
                // get the new values for the new subtab
                $cursubtab = $row['subtabid'];
                $cursubtabstring = $row['subtabstring'];
                $minsubblock = $cf->getMinSubblockID($fldarr, $cursubtab);
                $maxsubblock = $cf->getMaxSubblockID($fldarr, $cursubtab);
                
                // and output the headers for it
                $output .= '<div class="tab-page" id="tab-pane-' . $block . '-page-'.$sequence.'">
                            <h2 class="tab">'.$adb->query_result($result,$i,'subtabstring').'</h2>
                            <script type="text/javascript">tp' . $block . '.addTabPage( document.getElementById( "tab-pane-' . $block . '-page-'.$sequence.'" ) ); tab_to_id[next_tab_num] = ' . $sequence.'; next_tab_num++;</script>
                            <table width="100%" border="0" cellspacing="1" cellpadding="0">'."\n";
            }

            // get the new values for the new subblock
            $cursubblock = $row['subblockid'];
            $cursubblockstring = $row['subblockstring'];
            
            // and output the headers for it (each subblock is a row in the subtab table)
            $output .= "<tr><td>\n";
            $output .= '<table width="100%" border="0" cellspacing="1" cellpadding="0"><tr><th align="left" class="formSecHeader">'.$cursubblockstring.'</th></tr></table>'."\n";
	        $output .= '<table width="100%" border="0" cellspacing="1" cellpadding="2">'."\n";
            $firstinline = true;
        }    
		
		// fetch the rest of the fields
        $fieldtablename = $row['tablename'];	
        $fieldcolname = $row['columnname'];	
        $uitype = $row['uitype'];
        $fieldname = $row['fieldname'];
        $fieldlabel = $row['fieldlabel'];
        $maxlength = $row['maxlength'];
        $generatedtype = $row['generatedtype'];

        // if this is the first of the two, we have to start the table row
        if ( $firstinline ) {
            $output .= "<tr>\n";
        }
        
        // then comes the field in question
        if ( $mode == 'edit' ) {
            $custfld = getOutputHtml($uitype, $fieldname, $fieldlabel, $maxlength, $col_fields, $generatedtype);
        } else {
            $custfld = getDetailViewOutputHtml($uitype, $fieldname, $fieldlabel, $col_fields, $generatedtype);
        }
        $output .= $custfld . "\n";
        
		if ( $i == 0 )
		{
			$output .= $moveAddress;
		}
		
        // then, if this is the second in the line, then end the table row
        if ( !$firstinline ) {
            $output .= "</tr>\n";
        }
        
        $i++;
    }

    if ( $firstinline ) {
        $output .= "<td></td></tr>\n";
    }

    if ( $subblocking ) {
        $output .= "</table></td></tr>\n";
    }
    
    if ( $tabbing ) {
        $output .= "</table></div></div></td></tr>\n";
    } 

    return $output;

}

function getBlockTableHeader($header_label, $toggle=false, $toggleid='', $raw = false)
{
	global $mod_strings;
	$togglecode = '';
	if ( $toggle ) {
		$togglecode = '<img src="themes/images/toggle1.gif" id="img_'.$toggleid.'" width="15" height="15" onclick="javascript:expandCont(\''.$toggleid.'\');"/> ';
	}
	if ( $raw ) {
		$label = $header_label;
	} else {
		$label = $mod_strings[$header_label];
	}
	$output = '<table width="100%" border="0" cellspacing="1" cellpadding="0">';
	$output .= '<tr><th align="left" class="formSecHeader">'.$togglecode.$label.'</th></tr>';
	$output .= '</table>';
	return $output;

}


function getSubtabName($tab_id, $block_id, $subtab_id) {
  global $adb;
  if(strcmp($subtab_id, '') == 0) {
    return '';
  }
  $tab_id = intval($tab_id);
  $block_id = intval($block_id);
  $subtab_id = intval($subtab_id);
  $sql = "SELECT subtab.subtabstring FROM subtab WHERE tabid = $tab_id AND block = $block_id AND subtabid = $subtab_id";
  $result = $adb->query($sql);
  if($row = $result->FetchRow()) {
    $subtab_name = $row['subtabstring'];
  } else {
    $subtab_name = '';
  }
  return $subtab_name;
}

//added $viewid for customview to retain the sameview 27/5
function getTableHeaderNavigation($navigation_array, $url_qry,$module='',$action_val='index',$viewid='',$prefix_str='')
{
	global $theme;
	$theme_path="themes/".$theme."/";
	$image_path=$theme_path."images/";

	$record = isset($_REQUEST['record']) ? $_REQUEST['record'] : '';
	if ( $_REQUEST['module'] == $module ||  ($_REQUEST['module'] == 'Orders' &&  $module == 'SalesOrder')) {
		$relmodule = '';
	} else {
		$relmodule = $module;
	}
	//$relmodule = $_REQUEST['module'] == $module ? '' : $module;
	//$relmodule = '';
	$start = $relmodule.'start';
	$end_val = $relmodule.'end_val';
	$first = $relmodule.'first';
	$end = $relmodule.'end';
	$allflag = $relmodule.'allflag';
	$current = $relmodule.'current';
	$next = $relmodule.'next';
	$verylast = $relmodule.'verylast';
	$prev = $relmodule.'prev';
	$dir_name = $_REQUEST['module'];
	
	$output = '<td align="left">';
	$output .= $prefix_str;
	$output .= '</td>';
	$output .= '<td align="right">';
	$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'=1&viewname='.$viewid.'&'.$allflag.'='.$navigation_array[$allflag].'" >'.$navigation_array[$allflag].'</a>&nbsp;';
	if(($navigation_array[$prev]) != 0) {
		$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'=1&viewname='.$viewid.'" title="First"><img src="'.$image_path.'start.gif" border="0" align="absmiddle"></a>&nbsp;';
		$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'='.$navigation_array[$prev].'&viewname='.$viewid.'"><img src="'.$image_path.'previous.gif" border="0" align="absmiddle"></a>&nbsp;';

	} else {
		$output .= '<img src="'.$image_path.'start_disabled.gif" border="0" align="absmiddle">&nbsp;';
		$output .= '<img src="'.$image_path.'previous_disabled.gif" border="0" align="absmiddle">&nbsp;';
	}

	for ($i=$navigation_array[$first];$i<=$navigation_array[$end];$i++) {
		if ($navigation_array[$current]==$i) {
			$output .='<b>'.$i.'</b>&nbsp;';
		} else {
			$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'='.$i.'&viewname='.$viewid.'" >'.$i.'</a>&nbsp;';
		}
	}

	if(($navigation_array[$next]) !=0) {
		$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'='.$navigation_array[$next].'&viewname='.$viewid.'"><img src="'.$image_path.'next.gif" border="0" align="absmiddle"></a>&nbsp;';
		$output .= '<a href="index.php?module='.$dir_name.'&record='.$record.'&action='.$action_val.$url_qry.'&'.$start.'='.$navigation_array[$verylast].'&viewname='.$viewid.'"><img src="'.$image_path.'end.gif" border="0" align="absmiddle"></a>&nbsp;';
	} else {
		$output .= '<img src="'.$image_path.'next_disabled.gif" border="0" align="absmiddle">&nbsp;';
		$output .= '<img src="'.$image_path.'end_disabled.gif" border="0" align="absmiddle">&nbsp;';
	}

	$output .= '</td>';
	return $output; 
}

?>
